from .dataset import SentenceDataSet, BasicSentenceDataSet
from .wiktionary import WiktionaryDefinitionCorpus
from .codwoe import CODWOEDataSet
from .cpae import CPAEDataSet
from .allnli import AllNLIDataSet
from .stsb import STSBDataSet
from .entailmentbank import EntailmentBankDataSet
from .wordnet_filtered import WordNetFilteredDataSet
from .wordnet_spanish import SpanishWordNetFilteredDataSet
from .inference_types import InferenceTypesDataSet
from .complex_sentences import ComplexSentencesDataSet
from .yelp_st import YelpSTDataSet
from .snli import SNLIDataSet
from .yahoo import YahooAnswersDataSet



from typing import Callable
from saf import Vocabulary
from .dataset import SentenceDataSet, BASE_URL
from .wiktionary import WiktionaryDefinitionCorpus

PATH = "WordNet/wordnet_filtered_spacy_dsr_srl.jsonl.bz2"
URL = BASE_URL + "wordnet_filtered_spacy_dsr_srl.jsonl.bz2"

ANNOT_RESOURCES = {
    "pos+lemma+ctag+dep+dsr+srl": {
        "path": "WordNet/wordnet_filtered_spacy_dsr_srl.jsonl.bz2",
        "url": BASE_URL + "wordnet_filtered_spacy_dsr_srl.jsonl.bz2"
    }
}


class WordNetFilteredDataSet(SentenceDataSet):
    """
    WordNet filtered data set: A collection of filtered definition sentences from WordNet

    Each element of this dataset is a definition sentence (gloss) of a single synset, as it appears in WordNet.
    Definitions were filtered to only include those which definiendum (the term being defined) is representable by
    a single token from the LLaMa tokenizer.
    Each sentence is annotated with a "definiendum" and corresponding metadata from WordNet (WordNet id,
    brown_frequency, wordnet_frequency, category, abstraction_level, generalization_level).

    A pre-annotated instance can be obtained as follows:

    >>> dataset = WordNetFilteredDataSet.from_resource("pos+lemma+ctag+dep+dsr+srl")

    """
    def __init__(self, path: str = PATH, url: str = URL, **kwargs):
        super(WordNetFilteredDataSet, self).__init__(path, url, **kwargs)

    def vocabulary(self, source: str = "_token", lowercase: bool = True) -> Vocabulary:
        return WiktionaryDefinitionCorpus.vocabulary(self, source, lowercase)

    @classmethod
    def from_resource(cls, locator: str):
        """
        Downloads a pre-annotated resource available at the specified locator

        Example:
            >>> dataset = WordNetFilteredDataSet.from_resource("pos+lemma+ctag+dep+dsr+srl")
        """
        dataset = None
        if (locator in ANNOT_RESOURCES):
            dataset = cls(**ANNOT_RESOURCES[locator])
        else:
            print(f"No resource found at locator: {locator}")

        return dataset








from .dataset import SentenceDataSet, BASE_URL


PATH = "WordNet/Spanish_WordNet_filtered_data.jsonl.bz2"
URL = BASE_URL + "Spanish_WordNet_filtered_data.jsonl.bz2"


class SpanishWordNetFilteredDataSet(SentenceDataSet):
    """
        Spanish WordNet filtered data set: A collection of filtered definition sentences from the Spanish WordNet

        Each element of this dataset is a definition sentence (gloss) of a single synset, as it appears in WordNet.
        Definitions were filtered to only include those which definiendum (the term being defined) is representable by
        a single token from the LLaMa tokenizer.
        Each sentence is annotated with a "definiendum" and corresponding metadata from WordNet (WordNet id,
        cess_esp_frequencies, wordnet_frequency, category, abstraction_level, generalization_level).

    """
    def __init__(self, path: str = PATH, url: str = URL, **kwargs):
        super(SpanishWordNetFilteredDataSet, self).__init__(path, url, **kwargs)



import os
import gzip
from csv import DictReader
from tqdm import tqdm
from spacy.lang.en import English
from saf import Sentence, Token
from saf_datasets.annotators.spacy import SpacyAnnotator
from saf import Sentence, Vocabulary
from .dataset import SentenceDataSet, BASE_URL

PATH = "STSB/stsbenchmark.jsonl.bz2"
URL = BASE_URL + "stsbenchmark.jsonl.bz2"


class STSBDataSet(SentenceDataSet):
    """
    Wrapper for the STSB dataset (Cer et al. 2017): https://aclanthology.org/S17-2001/

    Sentence pairs are split into adjacent entries, sharing the same 'sid'.

    Sentence annotations: id, sid, split, genre, dataset
    """
    def __init__(self, path: str = PATH, url: str = URL, **kwargs):
        super(STSBDataSet, self).__init__(path, url, **kwargs)




from typing import Callable
from .dataset import SentenceDataSet, BASE_URL

FILE_VERSION = "entailment_trees_emnlp2021_data_v3"
PATH = "EntailmentBank/%s.jsonl.bz2" % FILE_VERSION
URL = BASE_URL + "%s.jsonl.bz2" % FILE_VERSION

ANNOT_RESOURCES = {
    "pos+lemma+ctag+dep+srl#expl_only-noreps": {
        "path": "EntailmentBank/eb_spacy_srl_explonly_noreps.jsonl.bz2",
        "url": BASE_URL + "eb_spacy_srl_explonly_noreps.jsonl.bz2"
    },
    "explanations+srl": {
        "path": "EntailmentBank/explanations.jsonl.bz2",
        "url": BASE_URL + "explanations.jsonl.bz2"
    }
}


class EntailmentBankDataSet(SentenceDataSet):
    """
    Wrapper for the EntailmentBank dataset: https://allenai.org/data/entailmentbank

    Context, hypothesis, question, answer and proof sentences for a single entry in the original dataset are split
    adjacently, and can be grouped by their 'id' annotation.

    Sentence annotations: id, task, split, type
    """
    def __init__(self, path: str = PATH, url: str = URL, **kwargs):
        super(EntailmentBankDataSet, self).__init__(path, url, **kwargs)

    @classmethod
    def from_resource(cls, locator: str):
        """
        Downloads a pre-annotated resource available at the specified locator

        Example:
            >>> dataset = EntailmentBankDataSet.from_resource("pos+lemma+ctag+dep+srl#expl_only-noreps")
        """
        dataset = None
        if (locator in ANNOT_RESOURCES):
            dataset = cls(**ANNOT_RESOURCES[locator])
        else:
            print(f"No resource found at locator: {locator}")

        return dataset




import pickle
import gzip
from typing import Callable
from csv import DictReader
from tqdm import tqdm
from spacy.lang.en import English
from saf import Sentence, Token
from .dataset import SentenceDataSet, BASE_URL

FILE_VERSION = "inftypes_v0.3"
PATH = "InferenceTypes/%s.jsonl.bz2" % FILE_VERSION
URL = BASE_URL + "%s.jsonl.bz2" % FILE_VERSION

ANNOT_RESOURCES = {
    "pos+lemma+ctag+dep+amr": {
        "path": "InferenceTypes/inftypes_v0.3.jsonl.bz2",
        "url": BASE_URL + "inftypes_v0.3.jsonl.bz2"
    }
}


class InferenceTypesDataSet(SentenceDataSet):
    """
    Wrapper for the Inference Types dataset, an annotated subset of
    the EntailmentBank: https://allenai.org/data/entailmentbank

    Premises and conclusion sentences for a single entry in the original dataset are split
    adjacently, and can be grouped by their 'id' annotation.

    Sentence annotations: id, role, type, new_type, type_amr_op
    """
    def __init__(self, path: str = PATH, url: str = URL, **kwargs):
        super(InferenceTypesDataSet, self).__init__(path, url, **kwargs)

    @classmethod
    def from_resource(cls, locator: str):
        """
        Downloads a pre-annotated resource available at the specified locator

        Example:
            >>> dataset = InferenceTypesDataSet.from_resource("pos+lemma+ctag+dep+amr")
        """
        dataset = None
        if (locator in ANNOT_RESOURCES):
            dataset = cls(**ANNOT_RESOURCES[locator])
        else:
            print(f"No resource found at locator: {locator}")

        return dataset




from .dataset import SentenceDataSet, BASE_URL

PATH = "Yahoo/yahoo_answers.jsonl.bz2"
URL = BASE_URL + "yahoo_answers.jsonl.bz2"


class YahooAnswersDataSet(SentenceDataSet):
    """
    Wrapper for the Yahoo Answers dataset (Zhang, Zhao, LeCun, 2015): https://proceedings.neurips.cc/paper/2015/hash/250cf8b51c773f3f8dc8b4be867a9a02-Abstract.html
    """
    def __init__(self, path: str = PATH, url: str = URL, **kwargs):
        super(YahooAnswersDataSet, self).__init__(path, url, **kwargs)



import torch
from torch import Tensor
from saf import Sentence
from .dataset import SentenceDataSet, BASE_URL

PATH = "CODWOE/CODWOE.jsonl.bz2"
URL = BASE_URL + "CODWOE.jsonl.bz2"


class CODWOEDataSet(SentenceDataSet):
    def __init__(self, path: str = PATH, url: str = URL, **kwargs):
        """
        Wrapper for the CODWOE dataset from Semeval-2022 Task 1 (Mickus et al. 2022): https://github.com/TimotheeMickus/codwoe
        """
        super(CODWOEDataSet, self).__init__(path, url, **kwargs)

    def __getitem__(self, idx: int) -> Sentence:
        """Fetches the ith definition in the dataset or all definitions for a given term.

        :param item: (int) for the ith definition in the dataset.
        :return: A single definition (Sentence).
        """
        sent = super().__getitem__(idx)
        sent.annotations["emb_char"] = torch.tensor(sent.annotations["emb_char"])
        sent.annotations["emb_electra"] = torch.tensor(sent.annotations["emb_electra"])
        sent.annotations["emb_sgns"] = torch.tensor(sent.annotations["emb_sgns"])

        return sent

    def embeddings(self, tag: str, device: str = "cpu") -> Tensor:
        embeddings = None
        annots = [self[i].annotations for i in range(len(self))]
        if (tag == "char"):
            embeddings = torch.stack([annot["emb_char"] for annot in annots])
        elif (tag == "electra"):
            embeddings = torch.stack([annot["emb_electra"] for annot in annots])
        elif (tag == "sgns"):
            embeddings = torch.stack([annot["emb_sgns"] for annot in annots])
        else:
            embeddings = super(CODWOEDataSet, self).embeddings(tag, device)

        return embeddings.to(device)


if __name__ == "__main__":
    defs = CODWOEDataSet()

    print(defs.vocabulary().freqs.most_common())

    exit(0)

    i = 0
    for sent in defs:
        print("Sent annotations:", sent.annotations)
        print("Token annotations:", [token.surface for token in sent.tokens])
        i += 1
        if (i > 10):
            break

    examples = [defs[5], defs[8]]
    annotator = SpacyAnnotator()
    annotator.annotate(examples)
    for example in examples:
        print(f"{example.annotations['id']} (\"{example.surface}\"): ")
        print(f"{[(token.surface, token.annotations) for token in example.tokens]}")

    print("Corpus size:", len(defs))



from typing import Callable
from .dataset import SentenceDataSet, BASE_URL

FILE_VERSION = "complex_sentences"
PATH = "ComplexSentences/%s.json.bz2" % FILE_VERSION
URL = BASE_URL + "%s.json.bz2" % FILE_VERSION


class ComplexSentencesDataSet(SentenceDataSet):
    """
    Wrapper for the Complex Sentences dataset (Niklaus et al.): https://aclanthology.org/P19-1333/

    Sentence annotations: source ["wiki", "newsela"], split
    Token annotations: label (see `ComplexSentencesDataSet.LABELS`)
    """

    LABELS = {
        "CNP": "coordinate noun phrases",
        "CVP":" coordinate verb phrases",
        "PreP": "prepositional phrase",
        "ParP": "participial phrase",
        "AP": "appositive phrase",
        "AM": "adverbial modifier",
        "CC": "coordinate clauses",
        "AC": "adverbial clause",
        "RC": "relative clause",
        "IDC": "(in)direct speech"
    }

    def __init__(self, path: str = PATH, url: str = URL, **kwargs):
        super(ComplexSentencesDataSet, self).__init__(path, url, **kwargs)




import bz2
from typing import Callable
from tqdm import tqdm
from spacy.lang.en import English
from saf import Token, Sentence, Vocabulary
from .dataset import SentenceDataSet, BASE_URL
from .wiktionary import WiktionaryDefinitionCorpus

PATH = "CPAE/cpae_definitions.jsonl.bz2"
URL = BASE_URL + "cpae_definitions.jsonl.bz2"


class CPAEDataSet(SentenceDataSet):
    """
    Wrapper for the CPAE dataset (Bosc, Vincent. 2018): https://aclanthology.org/D18-1181/
    """
    def __init__(self, path: str = PATH, url: str = URL, **kwargs):
        super(CPAEDataSet, self).__init__(path, url, **kwargs)

    def vocabulary(self, source: str = "_token", lowercase: bool = True) -> Vocabulary:
        return WiktionaryDefinitionCorpus.vocabulary(self, source, lowercase)



from .dataset import SentenceDataSet, BASE_URL

PATH = "SNLI/snli_1.0.jsonl.bz2"
URL = BASE_URL + "snli_1.0.jsonl.bz2"


class SNLIDataSet(SentenceDataSet):
    """
    Wrapper for the Stanford Natural Language Inference (SNLI) Corpus (Bowman et al., 2015): http://nlp.stanford.edu/pubs/snli_paper.pdf
    """
    def __init__(self, path: str = PATH, url: str = URL, **kwargs):
        super(SNLIDataSet, self).__init__(path, url, **kwargs)



from .dataset import SentenceDataSet, BASE_URL

PATH = "Yelp_ST/yelp_st.jsonl.bz2"
URL = BASE_URL + "yelp_st.jsonl.bz2"


class YelpSTDataSet(SentenceDataSet):
    """
    Wrapper for the Yelp Language Style Transfer dataset (Shen et al., 2017): https://proceedings.neurips.cc/paper_files/paper/2017/hash/2d2c8394e31101a261abf1784302bf75-Abstract.html
    """
    def __init__(self, path: str = PATH, url: str = URL, **kwargs):
        super(YelpSTDataSet, self).__init__(path, url, **kwargs)



import os
import gzip
from csv import DictReader
from tqdm import tqdm
from spacy.lang.en import English
from saf import Sentence, Token, Vocabulary
from .dataset import SentenceDataSet, BASE_URL

PATH = "AllNLI/AllNLI.jsonl.bz2"
URL = BASE_URL + "AllNLI.jsonl.bz2"


class AllNLIDataSet(SentenceDataSet):
    """
    Wrapper for the AllNLI dataset: https://www.sbert.net/examples/datasets/README.html

    Sentence annotations: id, dataset, split, label
    """
    def __init__(self, path: str = PATH, url: str = URL, **kwargs):
        super(AllNLIDataSet, self).__init__(path, url, **kwargs)

